#!/usr/bin/env python3
# -----------------------------------------------------------------------------
# Script developed by A. Saraiva-Souza, research scientist
# Description: Converts a CIF file to a simplified atoms.xyz and generates a
#              RESCU input file with preconfigured settings.
# -----------------------------------------------------------------------------

import argparse
from ase.io import read, write
import os
import numpy as np

def sort_by_z(atoms):
    """Returns a copy of atoms sorted by the z coordinate"""
    positions = atoms.get_positions()
    sorted_indices = np.argsort(positions[:, 2])  # sort by z-axis
    return atoms[sorted_indices]

def save_simple_xyz(atoms, xyz_file):
    """Saves atoms.xyz without extra columns"""
    write(xyz_file, atoms, format="xyz")

def generate_rescu_input(base_name, atoms):
    # Lattice vectors
    cell = atoms.get_cell()
    latvec = "\n".join(
        f"  [{row[0]:.12f} {row[1]:.12f} {row[2]:.12f}];"
        for row in cell
    )
    latvec = f"domain.latvec        = [\n{latvec[:-1]}\n]"

    # Unique elements and pseudopotential paths
    elements = sorted(set(atoms.get_chemical_symbols()))
    element_str = ""
    for i, el in enumerate(elements, 1):
        element_str += f"element({i}).species   = '{el}'\n"
        element_str += f"element({i}).path      = './pseudos/{el}_AtomicData.mat'\n"

    # Input file text
    input_txt = f"""%info.calculationType = 'self-consistent'
info.calculationType = 'relaxation'
sr.tol               = 1e-4
sr.moveableAtomList = [37:63]
sr.relaxedstructure = 'relaxedstructure.xyz'

info.savepath        = './results/{base_name}.mat'
info.outfile         = '{base_name}.out'



{latvec}

domain.boundary = [1 1 2]

LCAO.status           = 1
kpoint.sigma         = 0.05
eigensolver.emptyBand = 32
units.energy         = 'eV'
kpoint.gridn         = [2,2,1]
domain.lowres        = 0.35
functional.libxc      = true % required for vdw
functional.list      = {{'XC_GGA_X_OPTPBE_VDW','XC_GGA_C_OP_PBE'}}
option.maxSCFiteration = 200
mixing.type          = 'density'
mixing.method        = 'pulay'
mixing.tol           = [1e-05,1e-05]
mixing.maxhistory    = 20
spin.type            = 'degenerate'

{element_str}
atom.xyz             =  'atoms.xyz'
units.length         = 'Angstrom'
"""
    return input_txt

def main():
    parser = argparse.ArgumentParser(description="Convert CIF file to atoms.xyz and RESCU input file.")
    parser.add_argument("-i", "--input", required=True, help="Input .cif file")
    args = parser.parse_args()

    cif_path = args.input
    base_name = os.path.splitext(os.path.basename(cif_path))[0]

    # Read CIF using ASE
    atoms = read(cif_path)

    # Sort by increasing z coordinate
    sorted_atoms = sort_by_z(atoms)

    # Generate atoms.xyz
    save_simple_xyz(sorted_atoms, "atoms.xyz")
    print("✅ File 'atoms.xyz' generated (sorted by z).")

    # Generate RESCU input
    input_txt = generate_rescu_input(base_name, sorted_atoms)
    with open(f"{base_name}.input", "w") as f:
        f.write(input_txt)
    print(f"✅ RESCU input file '{base_name}.input' generated successfully.")

if __name__ == "__main__":
    main()
